@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2014 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -34,17 +34,17 @@ enum strm_status {
 };
 
 typedef struct _php_bz2_filter_data {
-	int persistent;
 	bz_stream strm;
 	char *inbuf;
-	size_t inbuf_len;
 	char *outbuf;
+	size_t inbuf_len;
 	size_t outbuf_len;
-	
-	/* Decompress options */
-	enum strm_status status;
-	unsigned int small_footprint : 1;
-	unsigned int expect_concatenated : 1;
+
+	enum strm_status status;              /* Decompress option */
+	unsigned int small_footprint : 1;     /* Decompress option */
+	unsigned int expect_concatenated : 1; /* Decompress option */
+
+	int persistent;
 } php_bz2_filter_data;
 
 /* }}} */
@@ -71,7 +71,7 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 	php_stream_bucket_brigade *buckets_out,
 	size_t *bytes_consumed,
 	int flags
-	TSRMLS_DC)
+	)
 {
 	php_bz2_filter_data *data;
 	php_stream_bucket *bucket;
@@ -80,24 +80,24 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 	php_stream_filter_status_t exit_status = PSFS_FEED_ME;
 	bz_stream *streamp;
 
-	if (!thisfilter || !thisfilter->abstract) {
+	if (!Z_PTR(thisfilter->abstract)) {
 		/* Should never happen */
 		return PSFS_ERR_FATAL;
 	}
 
-	data = (php_bz2_filter_data *)(thisfilter->abstract);
+	data = (php_bz2_filter_data *)Z_PTR(thisfilter->abstract);
 	streamp = &(data->strm);
 
 	while (buckets_in->head) {
 		size_t bin = 0, desired;
 
-		bucket = php_stream_bucket_make_writeable(buckets_in->head TSRMLS_CC);
+		bucket = php_stream_bucket_make_writeable(buckets_in->head);
 		while (bin < bucket->buflen) {
 			if (data->status == PHP_BZ2_UNITIALIZED) {
 				status = BZ2_bzDecompressInit(streamp, 0, data->small_footprint);
 
 				if (BZ_OK != status) {
-					php_stream_bucket_delref(bucket TSRMLS_CC);
+					php_stream_bucket_delref(bucket);
 					return PSFS_ERR_FATAL;
 				}
 
@@ -127,7 +127,7 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 				}
 			} else if (status != BZ_OK) {
 				/* Something bad happened */
-				php_stream_bucket_delref(bucket TSRMLS_CC);
+				php_stream_bucket_delref(bucket);
 				return PSFS_ERR_FATAL;
 			}
 			desired -= data->strm.avail_in; /* desired becomes what we consumed this round through */
@@ -139,19 +139,19 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 			if (data->strm.avail_out < data->outbuf_len) {
 				php_stream_bucket *out_bucket;
 				size_t bucketlen = data->outbuf_len - data->strm.avail_out;
-				out_bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0 TSRMLS_CC);
-				php_stream_bucket_append(buckets_out, out_bucket TSRMLS_CC);
+				out_bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0);
+				php_stream_bucket_append(buckets_out, out_bucket);
 				data->strm.avail_out = data->outbuf_len;
 				data->strm.next_out = data->outbuf;
 				exit_status = PSFS_PASS_ON;
 			} else if (status == BZ_STREAM_END && data->strm.avail_out >= data->outbuf_len) {
 				/* no more data to decompress, and nothing was spat out */
-				php_stream_bucket_delref(bucket TSRMLS_CC);
+				php_stream_bucket_delref(bucket);
 				return PSFS_PASS_ON;
 			}
 		}
 
-		php_stream_bucket_delref(bucket TSRMLS_CC);
+		php_stream_bucket_delref(bucket);
 	}
 
 	if ((data->status == PHP_BZ2_RUNNING) && (flags & PSFS_FLAG_FLUSH_CLOSE)) {
@@ -162,8 +162,8 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 			if (data->strm.avail_out < data->outbuf_len) {
 				size_t bucketlen = data->outbuf_len - data->strm.avail_out;
 
-				bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0 TSRMLS_CC);
-				php_stream_bucket_append(buckets_out, bucket TSRMLS_CC);
+				bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0);
+				php_stream_bucket_append(buckets_out, bucket);
 				data->strm.avail_out = data->outbuf_len;
 				data->strm.next_out = data->outbuf;
 				exit_status = PSFS_PASS_ON;
@@ -180,10 +180,10 @@ static php_stream_filter_status_t php_bz2_decompress_filter(
 	return exit_status;
 }
 
-static void php_bz2_decompress_dtor(php_stream_filter *thisfilter TSRMLS_DC)
+static void php_bz2_decompress_dtor(php_stream_filter *thisfilter)
 {
-	if (thisfilter && thisfilter->abstract) {
-		php_bz2_filter_data *data = thisfilter->abstract;
+	if (thisfilter && Z_PTR(thisfilter->abstract)) {
+		php_bz2_filter_data *data = Z_PTR(thisfilter->abstract);
 		if (data->status == PHP_BZ2_RUNNING) {
 			BZ2_bzDecompressEnd(&(data->strm));
 		}
@@ -209,7 +209,7 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 	php_stream_bucket_brigade *buckets_out,
 	size_t *bytes_consumed,
 	int flags
-	TSRMLS_DC)
+	)
 {
 	php_bz2_filter_data *data;
 	php_stream_bucket *bucket;
@@ -217,17 +217,17 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 	int status;
 	php_stream_filter_status_t exit_status = PSFS_FEED_ME;
 
-	if (!thisfilter || !thisfilter->abstract) {
+	if (!Z_PTR(thisfilter->abstract)) {
 		/* Should never happen */
 		return PSFS_ERR_FATAL;
 	}
 
-	data = (php_bz2_filter_data *)(thisfilter->abstract);
+	data = (php_bz2_filter_data *)Z_PTR(thisfilter->abstract);
 
 	while (buckets_in->head) {
 		size_t bin = 0, desired;
 
-		bucket = php_stream_bucket_make_writeable(buckets_in->head TSRMLS_CC);
+		bucket = php_stream_bucket_make_writeable(buckets_in->head);
 
 		while (bin < bucket->buflen) {
 			desired = bucket->buflen - bin;
@@ -240,7 +240,7 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 			status = BZ2_bzCompress(&(data->strm), flags & PSFS_FLAG_FLUSH_CLOSE ? BZ_FINISH : (flags & PSFS_FLAG_FLUSH_INC ? BZ_FLUSH : BZ_RUN));
 			if (status != BZ_RUN_OK && status != BZ_FLUSH_OK && status != BZ_FINISH_OK) {
 				/* Something bad happened */
-				php_stream_bucket_delref(bucket TSRMLS_CC);
+				php_stream_bucket_delref(bucket);
 				return PSFS_ERR_FATAL;
 			}
 			desired -= data->strm.avail_in; /* desired becomes what we consumed this round through */
@@ -253,14 +253,14 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 				php_stream_bucket *out_bucket;
 				size_t bucketlen = data->outbuf_len - data->strm.avail_out;
 
-				out_bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0 TSRMLS_CC);
-				php_stream_bucket_append(buckets_out, out_bucket TSRMLS_CC);
+				out_bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0);
+				php_stream_bucket_append(buckets_out, out_bucket);
 				data->strm.avail_out = data->outbuf_len;
 				data->strm.next_out = data->outbuf;
 				exit_status = PSFS_PASS_ON;
 			}
 		}
-		php_stream_bucket_delref(bucket TSRMLS_CC);
+		php_stream_bucket_delref(bucket);
 	}
 
 	if (flags & PSFS_FLAG_FLUSH_CLOSE) {
@@ -271,8 +271,8 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 			if (data->strm.avail_out < data->outbuf_len) {
 				size_t bucketlen = data->outbuf_len - data->strm.avail_out;
 
-				bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0 TSRMLS_CC);
-				php_stream_bucket_append(buckets_out, bucket TSRMLS_CC);
+				bucket = php_stream_bucket_new(stream, estrndup(data->outbuf, bucketlen), bucketlen, 1, 0);
+				php_stream_bucket_append(buckets_out, bucket);
 				data->strm.avail_out = data->outbuf_len;
 				data->strm.next_out = data->outbuf;
 				exit_status = PSFS_PASS_ON;
@@ -286,10 +286,10 @@ static php_stream_filter_status_t php_bz2_compress_filter(
 	return exit_status;
 }
 
-static void php_bz2_compress_dtor(php_stream_filter *thisfilter TSRMLS_DC)
+static void php_bz2_compress_dtor(php_stream_filter *thisfilter)
 {
-	if (thisfilter && thisfilter->abstract) {
-		php_bz2_filter_data *data = thisfilter->abstract;
+	if (Z_PTR(thisfilter->abstract)) {
+		php_bz2_filter_data *data = Z_PTR(thisfilter->abstract);
 		BZ2_bzCompressEnd(&(data->strm));
 		pefree(data->inbuf, data->persistent);
 		pefree(data->outbuf, data->persistent);
@@ -307,7 +307,7 @@ static php_stream_filter_ops php_bz2_compress_ops = {
 
 /* {{{ bzip2.* common factory */
 
-static php_stream_filter *php_bz2_filter_create(const char *filtername, zval *filterparams, int persistent TSRMLS_DC)
+static php_stream_filter *php_bz2_filter_create(const char *filtername, zval *filterparams, uint8_t persistent)
 {
 	php_stream_filter_ops *fops = NULL;
 	php_bz2_filter_data *data;
@@ -315,10 +315,6 @@ static php_stream_filter *php_bz2_filter_create(const char *filtername, zval *fi
 
 	/* Create this filter */
 	data = pecalloc(1, sizeof(php_bz2_filter_data), persistent);
-	if (!data) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed allocating %zu bytes", sizeof(php_bz2_filter_data));
-		return NULL;
-	}
 
 	/* Circular reference */
 	data->strm.opaque = (void *) data;
@@ -328,53 +324,29 @@ static php_stream_filter *php_bz2_filter_create(const char *filtername, zval *fi
 	data->persistent = persistent;
 	data->strm.avail_out = data->outbuf_len = data->inbuf_len = 2048;
 	data->strm.next_in = data->inbuf = (char *) pemalloc(data->inbuf_len, persistent);
-	if (!data->inbuf) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed allocating %zu bytes", data->inbuf_len);
-		pefree(data, persistent);
-		return NULL;
-	}
 	data->strm.avail_in = 0;
 	data->strm.next_out = data->outbuf = (char *) pemalloc(data->outbuf_len, persistent);
-	if (!data->outbuf) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Failed allocating %zu bytes", data->outbuf_len);
-		pefree(data->inbuf, persistent);
-		pefree(data, persistent);
-		return NULL;
-	}
 
 	if (strcasecmp(filtername, "bzip2.decompress") == 0) {
 		data->small_footprint = 0;
 		data->expect_concatenated = 0;
 
 		if (filterparams) {
-			zval **tmpzval = NULL;
+			zval *tmpzval = NULL;
 
 			if (Z_TYPE_P(filterparams) == IS_ARRAY || Z_TYPE_P(filterparams) == IS_OBJECT) {
-
-				if (SUCCESS == zend_hash_find(HASH_OF(filterparams), "concatenated", sizeof("concatenated"), (void **) &tmpzval) ) {
-					zval tmp, *tmp2;
-
-					tmp = **tmpzval;
-					zval_copy_ctor(&tmp);
-					tmp2 = &tmp;
-					convert_to_boolean_ex(&tmp2);
-					data->expect_concatenated = Z_LVAL(tmp);
+				if ((tmpzval = zend_hash_str_find(HASH_OF(filterparams), "concatenated", sizeof("concatenated")-1))) {
+					data->expect_concatenated = zend_is_true(tmpzval);
 					tmpzval = NULL;
 				}
 
-				zend_hash_find(HASH_OF(filterparams), "small", sizeof("small"), (void **) &tmpzval);
+				tmpzval = zend_hash_str_find(HASH_OF(filterparams), "small", sizeof("small")-1);
 			} else {
-				tmpzval = &filterparams;
+				tmpzval = filterparams;
 			}
 
 			if (tmpzval) {
-				zval tmp, *tmp2;
-
-				tmp = **tmpzval;
-				zval_copy_ctor(&tmp);
-				tmp2 = &tmp;
-				convert_to_boolean_ex(&tmp2);
-				data->small_footprint = Z_LVAL(tmp);
+				data->small_footprint = zend_is_true(tmpzval);
 			}
 		}
 
@@ -385,35 +357,26 @@ static php_stream_filter *php_bz2_filter_create(const char *filtername, zval *fi
 		int workFactor = PHP_BZ2_FILTER_DEFAULT_WORKFACTOR;
 
 		if (filterparams) {
-			zval **tmpzval;
+			zval *tmpzval;
 
 			if (Z_TYPE_P(filterparams) == IS_ARRAY || Z_TYPE_P(filterparams) == IS_OBJECT) {
-				if (zend_hash_find(HASH_OF(filterparams), "blocks", sizeof("blocks"), (void**) &tmpzval) == SUCCESS) {
+				if ((tmpzval = zend_hash_str_find(HASH_OF(filterparams), "blocks", sizeof("blocks")-1))) {
 					/* How much memory to allocate (1 - 9) x 100kb */
-					zval tmp;
-	
-					tmp = **tmpzval;
-					zval_copy_ctor(&tmp);
-					convert_to_long(&tmp);
-					if (Z_LVAL(tmp) < 1 || Z_LVAL(tmp) > 9) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid parameter given for number of blocks to allocate. (%ld)", Z_LVAL_PP(tmpzval));
+					zend_long blocks = zval_get_long(tmpzval);
+					if (blocks < 1 || blocks > 9) {
+						php_error_docref(NULL, E_WARNING, "Invalid parameter given for number of blocks to allocate. (" ZEND_LONG_FMT ")", blocks);
 					} else {
-						blockSize100k = Z_LVAL(tmp);
+						blockSize100k = (int) blocks;
 					}
 				}
 
-				if (zend_hash_find(HASH_OF(filterparams), "work", sizeof("work"), (void**) &tmpzval) == SUCCESS) {
+				if ((tmpzval = zend_hash_str_find(HASH_OF(filterparams), "work", sizeof("work")-1))) {
 					/* Work Factor (0 - 250) */
-					zval tmp;
-	
-					tmp = **tmpzval;
-					zval_copy_ctor(&tmp);
-					convert_to_long(&tmp);
-
-					if (Z_LVAL(tmp) < 0 || Z_LVAL(tmp) > 250) {
-						php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid parameter given for work factor. (%ld)", Z_LVAL(tmp));
+					zend_long work = zval_get_long(tmpzval);
+					if (work < 0 || work > 250) {
+						php_error_docref(NULL, E_WARNING, "Invalid parameter given for work factor. (" ZEND_LONG_FMT ")", work);
 					} else {
-						workFactor = Z_LVAL(tmp);
+						workFactor = (int) work;
 					}
 				}
 			}
