@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2012 The PHP Group                                |
+   | Copyright (c) 1997-2018 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -17,7 +17,7 @@
    +----------------------------------------------------------------------+
  */
 
-/* $Id: fopen_wrappers.c 321634 2012-01-01 13:15:04Z felipe $ */
+/* $Id$ */
 
 /* {{{ includes
  */
@@ -53,14 +53,8 @@
 #include <sys/socket.h>
 #endif
 
-#ifndef S_ISREG
-#define S_ISREG(mode)	(((mode) & S_IFMT) == S_IFREG)
-#endif
-
 #ifdef PHP_WIN32
 #include <winsock2.h>
-#elif defined(NETWARE) && defined(USE_WINSOCK)
-#include <novsock2.h>
 #else
 #include <netinet/in.h>
 #include <netdb.h>
@@ -69,7 +63,7 @@
 #endif
 #endif
 
-#if defined(PHP_WIN32) || defined(__riscos__) || defined(NETWARE)
+#if defined(PHP_WIN32) || defined(__riscos__)
 #undef AF_UNIX
 #endif
 
@@ -94,31 +88,31 @@ PHPAPI ZEND_INI_MH(OnUpdateBaseDir)
 
 	if (stage == PHP_INI_STAGE_STARTUP || stage == PHP_INI_STAGE_SHUTDOWN || stage == PHP_INI_STAGE_ACTIVATE || stage == PHP_INI_STAGE_DEACTIVATE) {
 		/* We're in a PHP_INI_SYSTEM context, no restrictions */
-		*p = new_value;
+		*p = new_value ? ZSTR_VAL(new_value) : NULL;
 		return SUCCESS;
 	}
 
 	/* Otherwise we're in runtime */
 	if (!*p || !**p) {
 		/* open_basedir not set yet, go ahead and give it a value */
-		*p = new_value;
+		*p = ZSTR_VAL(new_value);
 		return SUCCESS;
 	}
 
 	/* Shortcut: When we have a open_basedir and someone tries to unset, we know it'll fail */
-	if (!new_value || !*new_value) {
+	if (!new_value || !*ZSTR_VAL(new_value)) {
 		return FAILURE;
 	}
 
 	/* Is the proposed open_basedir at least as restrictive as the current setting? */
-	ptr = pathbuf = estrdup(new_value);
+	ptr = pathbuf = estrdup(ZSTR_VAL(new_value));
 	while (ptr && *ptr) {
 		end = strchr(ptr, DEFAULT_DIR_SEPARATOR);
 		if (end != NULL) {
 			*end = '\0';
 			end++;
 		}
-		if (php_check_open_basedir_ex(ptr, 0 TSRMLS_CC) != 0) {
+		if (php_check_open_basedir_ex(ptr, 0) != 0) {
 			/* At least one portion of this open_basedir is less restrictive than the prior one, FAIL */
 			efree(pathbuf);
 			return FAILURE;
@@ -128,7 +122,7 @@ PHPAPI ZEND_INI_MH(OnUpdateBaseDir)
 	efree(pathbuf);
 
 	/* Everything checks out, set it */
-	*p = new_value;
+	*p = ZSTR_VAL(new_value);
 
 	return SUCCESS;
 }
@@ -139,16 +133,16 @@ PHPAPI ZEND_INI_MH(OnUpdateBaseDir)
 	open_basedir. Returns -1 if error or not in the open_basedir, else 0.
 	When open_basedir is NULL, always return 0.
 */
-PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path TSRMLS_DC)
+PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path)
 {
 	char resolved_name[MAXPATHLEN];
 	char resolved_basedir[MAXPATHLEN];
 	char local_open_basedir[MAXPATHLEN];
 	char path_tmp[MAXPATHLEN];
 	char *path_file;
-	int resolved_basedir_len;
-	int resolved_name_len;
-	int path_len;
+	size_t resolved_basedir_len;
+	size_t resolved_name_len;
+	size_t path_len;
 	int nesting_level = 0;
 
 	/* Special case basedir==".": Use script-directory */
@@ -164,7 +158,7 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 	}
 
 	/* normalize and expand path */
-	if (expand_filepath(path, resolved_name TSRMLS_CC) == NULL) {
+	if (expand_filepath(path, resolved_name) == NULL) {
 		return -1;
 	}
 
@@ -173,28 +167,22 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 
 	while (VCWD_REALPATH(path_tmp, resolved_name) == NULL) {
 #if defined(PHP_WIN32) || defined(HAVE_SYMLINK)
-#if defined(PHP_WIN32)
-		if (EG(windows_version_info).dwMajorVersion > 5) {
-#endif
-			if (nesting_level == 0) {
-				int ret;
-				char buf[MAXPATHLEN];
+		if (nesting_level == 0) {
+			int ret;
+			char buf[MAXPATHLEN];
 
-				ret = php_sys_readlink(path_tmp, buf, MAXPATHLEN - 1);
-				if (ret < 0) {
-					/* not a broken symlink, move along.. */
-				} else {
-					/* put the real path into the path buffer */
-					memcpy(path_tmp, buf, ret);
-					path_tmp[ret] = '\0';
-				}
+			ret = php_sys_readlink(path_tmp, buf, MAXPATHLEN - 1);
+			if (ret < 0) {
+				/* not a broken symlink, move along.. */
+			} else {
+				/* put the real path into the path buffer */
+				memcpy(path_tmp, buf, ret);
+				path_tmp[ret] = '\0';
 			}
-#if defined(PHP_WIN32)
 		}
 #endif
-#endif
 
-#if defined(PHP_WIN32) || defined(NETWARE)
+#ifdef PHP_WIN32
 		path_file = strrchr(path_tmp, DEFAULT_SLASH);
 		if (!path_file) {
 			path_file = strrchr(path_tmp, '/');
@@ -207,11 +195,11 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 			return -1;
 		} else {
 			path_len = path_file - path_tmp + 1;
-#if defined(PHP_WIN32) || defined(NETWARE)
+#ifdef PHP_WIN32
 			if (path_len > 1 && path_tmp[path_len - 2] == ':') {
 				if (path_len != 3) {
 					return -1;
-				} 
+				}
 				/* this is c:\ */
 				path_tmp[path_len] = '\0';
 			} else {
@@ -225,13 +213,14 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 	}
 
 	/* Resolve open_basedir to resolved_basedir */
-	if (expand_filepath(local_open_basedir, resolved_basedir TSRMLS_CC) != NULL) {
+	if (expand_filepath(local_open_basedir, resolved_basedir) != NULL) {
+		size_t basedir_len = strlen(basedir);
 		/* Handler for basedirs that end with a / */
 		resolved_basedir_len = strlen(resolved_basedir);
-#if defined(PHP_WIN32) || defined(NETWARE)
-		if (basedir[strlen(basedir) - 1] == PHP_DIR_SEPARATOR || basedir[strlen(basedir) - 1] == '/') {
+#ifdef PHP_WIN32
+		if (basedir[basedir_len - 1] == PHP_DIR_SEPARATOR || basedir[basedir_len - 1] == '/') {
 #else
-		if (basedir[strlen(basedir) - 1] == PHP_DIR_SEPARATOR) {
+		if (basedir[basedir_len - 1] == PHP_DIR_SEPARATOR) {
 #endif
 			if (resolved_basedir[resolved_basedir_len - 1] != PHP_DIR_SEPARATOR) {
 				resolved_basedir[resolved_basedir_len] = PHP_DIR_SEPARATOR;
@@ -251,7 +240,7 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 		}
 
 		/* Check the path */
-#if defined(PHP_WIN32) || defined(NETWARE)
+#ifdef PHP_WIN32
 		if (strncasecmp(resolved_basedir, resolved_name, resolved_basedir_len) == 0) {
 #else
 		if (strncmp(resolved_basedir, resolved_name, resolved_basedir_len) == 0) {
@@ -266,7 +255,7 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 		} else {
 			/* /openbasedir/ and /openbasedir are the same directory */
 			if (resolved_basedir_len == (resolved_name_len + 1) && resolved_basedir[resolved_basedir_len - 1] == PHP_DIR_SEPARATOR) {
-#if defined(PHP_WIN32) || defined(NETWARE)
+#ifdef PHP_WIN32
 				if (strncasecmp(resolved_basedir, resolved_name, resolved_name_len) == 0) {
 #else
 				if (strncmp(resolved_basedir, resolved_name, resolved_name_len) == 0) {
@@ -283,14 +272,14 @@ PHPAPI int php_check_specific_open_basedir(const char *basedir, const char *path
 }
 /* }}} */
 
-PHPAPI int php_check_open_basedir(const char *path TSRMLS_DC)
+PHPAPI int php_check_open_basedir(const char *path)
 {
-	return php_check_open_basedir_ex(path, 1 TSRMLS_CC);
+	return php_check_open_basedir_ex(path, 1);
 }
 
 /* {{{ php_check_open_basedir
  */
-PHPAPI int php_check_open_basedir_ex(const char *path, int warn TSRMLS_DC)
+PHPAPI int php_check_open_basedir_ex(const char *path, int warn)
 {
 	/* Only check when open_basedir is available */
 	if (PG(open_basedir) && *PG(open_basedir)) {
@@ -301,7 +290,7 @@ PHPAPI int php_check_open_basedir_ex(const char *path, int warn TSRMLS_DC)
 		/* Check if the path is too long so we can give a more useful error
 		* message. */
 		if (strlen(path) > (MAXPATHLEN - 1)) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "File name is longer than the maximum allowed path length on this platform (%d): %s", MAXPATHLEN, path);
+			php_error_docref(NULL, E_WARNING, "File name is longer than the maximum allowed path length on this platform (%d): %s", MAXPATHLEN, path);
 			errno = EINVAL;
 			return -1;
 		}
@@ -317,7 +306,7 @@ PHPAPI int php_check_open_basedir_ex(const char *path, int warn TSRMLS_DC)
 				end++;
 			}
 
-			if (php_check_specific_open_basedir(ptr, path TSRMLS_CC) == 0) {
+			if (php_check_specific_open_basedir(ptr, path) == 0) {
 				efree(pathbuf);
 				return 0;
 			}
@@ -325,7 +314,7 @@ PHPAPI int php_check_open_basedir_ex(const char *path, int warn TSRMLS_DC)
 			ptr = end;
 		}
 		if (warn) {
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "open_basedir restriction in effect. File(%s) is not within the allowed path(s): (%s)", path, PG(open_basedir));
+			php_error_docref(NULL, E_WARNING, "open_basedir restriction in effect. File(%s) is not within the allowed path(s): (%s)", path, PG(open_basedir));
 		}
 		efree(pathbuf);
 		errno = EPERM; /* we deny permission to open it */
@@ -339,16 +328,21 @@ PHPAPI int php_check_open_basedir_ex(const char *path, int warn TSRMLS_DC)
 
 /* {{{ php_fopen_and_set_opened_path
  */
-static FILE *php_fopen_and_set_opened_path(const char *path, const char *mode, char **opened_path TSRMLS_DC)
+static FILE *php_fopen_and_set_opened_path(const char *path, const char *mode, zend_string **opened_path)
 {
 	FILE *fp;
 
-	if (php_check_open_basedir((char *)path TSRMLS_CC)) {
+	if (php_check_open_basedir((char *)path)) {
 		return NULL;
 	}
 	fp = VCWD_FOPEN(path, mode);
 	if (fp && opened_path) {
-		*opened_path = expand_filepath_with_mode(path, NULL, NULL, 0, CWD_EXPAND TSRMLS_CC);
+		//TODO :avoid reallocation
+		char *tmp = expand_filepath_with_mode(path, NULL, NULL, 0, CWD_EXPAND);
+		if (tmp) {
+			*opened_path = zend_string_init(tmp, strlen(tmp), 0);
+			efree(tmp);
+		}
 	}
 	return fp;
 }
@@ -356,11 +350,11 @@ static FILE *php_fopen_and_set_opened_path(const char *path, const char *mode, c
 
 /* {{{ php_fopen_primary_script
  */
-PHPAPI int php_fopen_primary_script(zend_file_handle *file_handle TSRMLS_DC)
+PHPAPI int php_fopen_primary_script(zend_file_handle *file_handle)
 {
 	char *path_info;
 	char *filename = NULL;
-	char *resolved_path = NULL;
+	zend_string *resolved_path = NULL;
 	int length;
 	zend_bool orig_display_errors;
 
@@ -401,65 +395,73 @@ PHPAPI int php_fopen_primary_script(zend_file_handle *file_handle TSRMLS_DC)
 				spprintf(&filename, 0, "%s%c%s%c%s", pw->pw_dir, PHP_DIR_SEPARATOR, PG(user_dir), PHP_DIR_SEPARATOR, s + 1); /* Safe */
 			} else {
 				filename = SG(request_info).path_translated;
-			} 
+			}
 #if defined(ZTS) && defined(HAVE_GETPWNAM_R) && defined(_SC_GETPW_R_SIZE_MAX)
 			efree(pwbuf);
 #endif
 		}
 	} else
 #endif
-	if (PG(doc_root) && path_info && (length = strlen(PG(doc_root))) &&
+	if (PG(doc_root) && path_info && (length = (int)strlen(PG(doc_root))) &&
 		IS_ABSOLUTE_PATH(PG(doc_root), length)) {
-		int path_len = strlen(path_info);
+		int path_len = (int)strlen(path_info);
 		filename = emalloc(length + path_len + 2);
-		if (filename) {
-			memcpy(filename, PG(doc_root), length);
-			if (!IS_SLASH(filename[length - 1])) {	/* length is never 0 */
-				filename[length++] = PHP_DIR_SEPARATOR;
-			}
-			if (IS_SLASH(path_info[0])) {
-				length--;
-			}
-			strncpy(filename + length, path_info, path_len + 1);
+		memcpy(filename, PG(doc_root), length);
+		if (!IS_SLASH(filename[length - 1])) {	/* length is never 0 */
+			filename[length++] = PHP_DIR_SEPARATOR;
 		}
+		if (IS_SLASH(path_info[0])) {
+			length--;
+		}
+		strncpy(filename + length, path_info, path_len + 1);
 	} else {
 		filename = SG(request_info).path_translated;
 	}
 
 
 	if (filename) {
-		resolved_path = zend_resolve_path(filename, strlen(filename) TSRMLS_CC);
+		resolved_path = zend_resolve_path(filename, (int)strlen(filename));
 	}
 
 	if (!resolved_path) {
 		if (SG(request_info).path_translated != filename) {
-			STR_FREE(filename);
+			if (filename) {
+				efree(filename);
+			}
 		}
 		/* we have to free SG(request_info).path_translated here because
 		 * php_destroy_request_info assumes that it will get
 		 * freed when the include_names hash is emptied, but
 		 * we're not adding it in this case */
-		STR_FREE(SG(request_info).path_translated);
-		SG(request_info).path_translated = NULL;
+		if (SG(request_info).path_translated) {
+			efree(SG(request_info).path_translated);
+			SG(request_info).path_translated = NULL;
+		}
 		return FAILURE;
 	}
-	efree(resolved_path);
+	zend_string_release(resolved_path);
 
 	orig_display_errors = PG(display_errors);
 	PG(display_errors) = 0;
-	if (zend_stream_open(filename, file_handle TSRMLS_CC) == FAILURE) {
+	if (zend_stream_open(filename, file_handle) == FAILURE) {
 		PG(display_errors) = orig_display_errors;
 		if (SG(request_info).path_translated != filename) {
-			STR_FREE(filename);
+			if (filename) {
+				efree(filename);
+			}
+		}
+		if (SG(request_info).path_translated) {
+			efree(SG(request_info).path_translated);
+			SG(request_info).path_translated = NULL;
 		}
-		STR_FREE(SG(request_info).path_translated);	/* for same reason as above */
-		SG(request_info).path_translated = NULL;
 		return FAILURE;
 	}
 	PG(display_errors) = orig_display_errors;
 
 	if (SG(request_info).path_translated != filename) {
-		STR_FREE(SG(request_info).path_translated);	/* for same reason as above */
+		if (SG(request_info).path_translated) {
+			efree(SG(request_info).path_translated);
+		}
 		SG(request_info).path_translated = filename;
 	}
 
@@ -470,13 +472,14 @@ PHPAPI int php_fopen_primary_script(zend_file_handle *file_handle TSRMLS_DC)
 /* {{{ php_resolve_path
  * Returns the realpath for given filename according to include path
  */
-PHPAPI char *php_resolve_path(const char *filename, int filename_length, const char *path TSRMLS_DC)
+PHPAPI zend_string *php_resolve_path(const char *filename, int filename_length, const char *path)
 {
 	char resolved_path[MAXPATHLEN];
 	char trypath[MAXPATHLEN];
 	const char *ptr, *end, *p;
-	char *actual_path;
+	const char *actual_path;
 	php_stream_wrapper *wrapper;
+	zend_string *exec_filename;
 
 	if (!filename || CHECK_NULL_PATH(filename, filename_length)) {
 		return NULL;
@@ -485,23 +488,30 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
 	/* Don't resolve paths which contain protocol (except of file://) */
 	for (p = filename; isalnum((int)*p) || *p == '+' || *p == '-' || *p == '.'; p++);
 	if ((*p == ':') && (p - filename > 1) && (p[1] == '/') && (p[2] == '/')) {
-		wrapper = php_stream_locate_url_wrapper(filename, &actual_path, STREAM_OPEN_FOR_INCLUDE TSRMLS_CC);
+		wrapper = php_stream_locate_url_wrapper(filename, &actual_path, STREAM_OPEN_FOR_INCLUDE);
 		if (wrapper == &php_plain_files_wrapper) {
-			if (tsrm_realpath(actual_path, resolved_path TSRMLS_CC)) {
-				return estrdup(resolved_path);
+			if (tsrm_realpath(actual_path, resolved_path)) {
+				return zend_string_init(resolved_path, strlen(resolved_path), 0);
 			}
 		}
 		return NULL;
 	}
 
-	if ((*filename == '.' && 
-	     (IS_SLASH(filename[1]) || 
+	if ((*filename == '.' &&
+	     (IS_SLASH(filename[1]) ||
 	      ((filename[1] == '.') && IS_SLASH(filename[2])))) ||
 	    IS_ABSOLUTE_PATH(filename, filename_length) ||
+#ifdef PHP_WIN32
+		/* This should count as an absolute local path as well, however
+		   IS_ABSOLUTE_PATH doesn't care about this path form till now. It
+		   might be a big thing to extend, thus just a local handling for
+		   now. */
+		filename_length >=2 && IS_SLASH(filename[0]) && !IS_SLASH(filename[1]) ||
+#endif
 	    !path ||
 	    !*path) {
-		if (tsrm_realpath(filename, resolved_path TSRMLS_CC)) {
-			return estrdup(resolved_path);
+		if (tsrm_realpath(filename, resolved_path)) {
+			return zend_string_init(resolved_path, strlen(resolved_path), 0);
 		} else {
 			return NULL;
 		}
@@ -522,7 +532,7 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
 		}
 		end = strchr(p, DEFAULT_DIR_SEPARATOR);
 		if (end) {
-			if ((end-ptr) + 1 + filename_length + 1 >= MAXPATHLEN) {
+			if (filename_length > (MAXPATHLEN - 2) || (end-ptr) > MAXPATHLEN || (end-ptr) + 1 + (size_t)filename_length + 1 >= MAXPATHLEN) {
 				ptr = end + 1;
 				continue;
 			}
@@ -531,9 +541,9 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
 			memcpy(trypath+(end-ptr)+1, filename, filename_length+1);
 			ptr = end+1;
 		} else {
-			int len = strlen(ptr);
+			size_t len = strlen(ptr);
 
-			if (len + 1 + filename_length + 1 >= MAXPATHLEN) {
+			if (filename_length > (MAXPATHLEN - 2) || len > MAXPATHLEN || len + 1 + (size_t)filename_length + 1 >= MAXPATHLEN) {
 				break;
 			}
 			memcpy(trypath, ptr, len);
@@ -543,34 +553,35 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
 		}
 		actual_path = trypath;
 		if (is_stream_wrapper) {
-			wrapper = php_stream_locate_url_wrapper(trypath, &actual_path, STREAM_OPEN_FOR_INCLUDE TSRMLS_CC);
+			wrapper = php_stream_locate_url_wrapper(trypath, &actual_path, STREAM_OPEN_FOR_INCLUDE);
 			if (!wrapper) {
 				continue;
 			} else if (wrapper != &php_plain_files_wrapper) {
 				if (wrapper->wops->url_stat) {
 					php_stream_statbuf ssb;
 
-					if (SUCCESS == wrapper->wops->url_stat(wrapper, trypath, 0, &ssb, NULL TSRMLS_CC)) {
-						return estrdup(trypath);
+					if (SUCCESS == wrapper->wops->url_stat(wrapper, trypath, 0, &ssb, NULL)) {
+						return zend_string_init(trypath, strlen(trypath), 0);
 					}
 				}
 				continue;
 			}
 		}
-		if (tsrm_realpath(actual_path, resolved_path TSRMLS_CC)) {
-			return estrdup(resolved_path);
+		if (tsrm_realpath(actual_path, resolved_path)) {
+			return zend_string_init(resolved_path, strlen(resolved_path), 0);
 		}
 	} /* end provided path */
 
 	/* check in calling scripts' current working directory as a fall back case
 	 */
-	if (zend_is_executing(TSRMLS_C)) {
-		const char *exec_fname = zend_get_executed_filename(TSRMLS_C);
-		int exec_fname_length = strlen(exec_fname);
+	if (zend_is_executing() &&
+	    (exec_filename = zend_get_executed_filename_ex()) != NULL) {
+		const char *exec_fname = ZSTR_VAL(exec_filename);
+		size_t exec_fname_length = ZSTR_LEN(exec_filename);
 
-		while ((--exec_fname_length >= 0) && !IS_SLASH(exec_fname[exec_fname_length]));
-		if (exec_fname && exec_fname[0] != '[' &&
-		    exec_fname_length > 0 &&
+		while ((--exec_fname_length < SIZE_MAX) && !IS_SLASH(exec_fname[exec_fname_length]));
+		if (exec_fname_length > 0 &&
+			filename_length < (MAXPATHLEN - 2) &&
 		    exec_fname_length + 1 + filename_length + 1 < MAXPATHLEN) {
 			memcpy(trypath, exec_fname, exec_fname_length + 1);
 			memcpy(trypath+exec_fname_length + 1, filename, filename_length+1);
@@ -579,23 +590,23 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
 			/* Check for stream wrapper */
 			for (p = trypath; isalnum((int)*p) || *p == '+' || *p == '-' || *p == '.'; p++);
 			if ((*p == ':') && (p - trypath > 1) && (p[1] == '/') && (p[2] == '/')) {
-				wrapper = php_stream_locate_url_wrapper(trypath, &actual_path, STREAM_OPEN_FOR_INCLUDE TSRMLS_CC);
+				wrapper = php_stream_locate_url_wrapper(trypath, &actual_path, STREAM_OPEN_FOR_INCLUDE);
 				if (!wrapper) {
 					return NULL;
 				} else if (wrapper != &php_plain_files_wrapper) {
 					if (wrapper->wops->url_stat) {
 						php_stream_statbuf ssb;
 
-						if (SUCCESS == wrapper->wops->url_stat(wrapper, trypath, 0, &ssb, NULL TSRMLS_CC)) {
-							return estrdup(trypath);
+						if (SUCCESS == wrapper->wops->url_stat(wrapper, trypath, 0, &ssb, NULL)) {
+							return zend_string_init(trypath, strlen(trypath), 0);
 						}
 					}
 					return NULL;
 				}
 			}
 
-			if (tsrm_realpath(actual_path, resolved_path TSRMLS_CC)) {
-				return estrdup(resolved_path);
+			if (tsrm_realpath(actual_path, resolved_path)) {
+				return zend_string_init(resolved_path, strlen(resolved_path), 0);
 			}
 		}
 	}
@@ -608,15 +619,13 @@ PHPAPI char *php_resolve_path(const char *filename, int filename_length, const c
  * Tries to open a file with a PATH-style list of directories.
  * If the filename starts with "." or "/", the path is ignored.
  */
-PHPAPI FILE *php_fopen_with_path(const char *filename, const char *mode, const char *path, char **opened_path TSRMLS_DC)
+PHPAPI FILE *php_fopen_with_path(const char *filename, const char *mode, const char *path, zend_string **opened_path)
 {
 	char *pathbuf, *ptr, *end;
-	const char *exec_fname;
 	char trypath[MAXPATHLEN];
 	FILE *fp;
-	int path_length;
 	int filename_length;
-	int exec_fname_length;
+	zend_string *exec_filename;
 
 	if (opened_path) {
 		*opened_path = NULL;
@@ -626,31 +635,36 @@ PHPAPI FILE *php_fopen_with_path(const char *filename, const char *mode, const c
 		return NULL;
 	}
 
-	filename_length = strlen(filename);
+	filename_length = (int)strlen(filename);
+#ifndef PHP_WIN32
+	(void) filename_length;
+#endif
 
 	/* Relative path open */
 	if ((*filename == '.')
 	/* Absolute path open */
 	 || IS_ABSOLUTE_PATH(filename, filename_length)
-	 || (!path || (path && !*path))
+	 || (!path || !*path)
 	) {
-		return php_fopen_and_set_opened_path(filename, mode, opened_path TSRMLS_CC);
+		return php_fopen_and_set_opened_path(filename, mode, opened_path);
 	}
 
 	/* check in provided path */
 	/* append the calling scripts' current working directory
 	 * as a fall back case
 	 */
-	if (zend_is_executing(TSRMLS_C)) {
-		exec_fname = zend_get_executed_filename(TSRMLS_C);
-		exec_fname_length = strlen(exec_fname);
-		path_length = strlen(path);
+	if (zend_is_executing() &&
+	    (exec_filename = zend_get_executed_filename_ex()) != NULL) {
+		const char *exec_fname = ZSTR_VAL(exec_filename);
+		size_t exec_fname_length = ZSTR_LEN(exec_filename);
 
-		while ((--exec_fname_length >= 0) && !IS_SLASH(exec_fname[exec_fname_length]));
+		while ((--exec_fname_length < SIZE_MAX) && !IS_SLASH(exec_fname[exec_fname_length]));
 		if ((exec_fname && exec_fname[0] == '[') || exec_fname_length <= 0) {
 			/* [no active file] or no path */
 			pathbuf = estrdup(path);
 		} else {
+			size_t path_length = strlen(path);
+
 			pathbuf = (char *) emalloc(exec_fname_length + path_length + 1 + 1);
 			memcpy(pathbuf, path, path_length);
 			pathbuf[path_length] = DEFAULT_DIR_SEPARATOR;
@@ -670,9 +684,9 @@ PHPAPI FILE *php_fopen_with_path(const char *filename, const char *mode, const c
 			end++;
 		}
 		if (snprintf(trypath, MAXPATHLEN, "%s/%s", ptr, filename) >= MAXPATHLEN) {
-			php_error_docref(NULL TSRMLS_CC, E_NOTICE, "%s/%s path was truncated to %d", ptr, filename, MAXPATHLEN);
+			php_error_docref(NULL, E_NOTICE, "%s/%s path was truncated to %d", ptr, filename, MAXPATHLEN);
 		}
-		fp = php_fopen_and_set_opened_path(trypath, mode, opened_path TSRMLS_CC);
+		fp = php_fopen_and_set_opened_path(trypath, mode, opened_path);
 		if (fp) {
 			efree(pathbuf);
 			return fp;
@@ -727,31 +741,36 @@ PHPAPI char *php_strip_url_passwd(char *url)
 
 /* {{{ expand_filepath
  */
-PHPAPI char *expand_filepath(const char *filepath, char *real_path TSRMLS_DC)
+PHPAPI char *expand_filepath(const char *filepath, char *real_path)
 {
-	return expand_filepath_ex(filepath, real_path, NULL, 0 TSRMLS_CC);
+	return expand_filepath_ex(filepath, real_path, NULL, 0);
 }
 /* }}} */
 
 /* {{{ expand_filepath_ex
  */
-PHPAPI char *expand_filepath_ex(const char *filepath, char *real_path, const char *relative_to, size_t relative_to_len TSRMLS_DC)
+PHPAPI char *expand_filepath_ex(const char *filepath, char *real_path, const char *relative_to, size_t relative_to_len)
 {
-	return expand_filepath_with_mode(filepath, real_path, relative_to, relative_to_len, CWD_FILEPATH TSRMLS_CC);
+	return expand_filepath_with_mode(filepath, real_path, relative_to, relative_to_len, CWD_FILEPATH);
 }
 /* }}} */
 
 /* {{{ expand_filepath_use_realpath
  */
-PHPAPI char *expand_filepath_with_mode(const char *filepath, char *real_path, const char *relative_to, size_t relative_to_len, int realpath_mode TSRMLS_DC)
+PHPAPI char *expand_filepath_with_mode(const char *filepath, char *real_path, const char *relative_to, size_t relative_to_len, int realpath_mode)
 {
 	cwd_state new_state;
 	char cwd[MAXPATHLEN];
 	int copy_len;
+	int path_len;
 
 	if (!filepath[0]) {
 		return NULL;
-	} else if (IS_ABSOLUTE_PATH(filepath, strlen(filepath))) {
+	}
+
+	path_len = (int)strlen(filepath);
+
+	if (IS_ABSOLUTE_PATH(filepath, path_len)) {
 		cwd[0] = '\0';
 	} else {
 		const char *iam = SG(request_info).path_translated;
@@ -774,8 +793,13 @@ PHPAPI char *expand_filepath_with_mode(const char *filepath, char *real_path, co
 				/* return a relative file path if for any reason
 				 * we cannot cannot getcwd() and the requested,
 				 * relatively referenced file is accessible */
-				copy_len = strlen(filepath) > MAXPATHLEN - 1 ? MAXPATHLEN - 1 : strlen(filepath);
-				real_path = estrndup(filepath, copy_len);
+				copy_len = path_len > MAXPATHLEN - 1 ? MAXPATHLEN - 1 : path_len;
+				if (real_path) {
+					memcpy(real_path, filepath, copy_len);
+					real_path[copy_len] = '\0';
+				} else {
+					real_path = estrndup(filepath, copy_len);
+				}
 				close(fdtest);
 				return real_path;
 			} else {
@@ -786,11 +810,11 @@ PHPAPI char *expand_filepath_with_mode(const char *filepath, char *real_path, co
 		}
 	}
 
-	new_state.cwd = strdup(cwd);
-	new_state.cwd_length = strlen(cwd);
+	new_state.cwd = estrdup(cwd);
+	new_state.cwd_length = (int)strlen(cwd);
 
-	if (virtual_file_ex(&new_state, filepath, NULL, realpath_mode TSRMLS_CC)) {
-		free(new_state.cwd);
+	if (virtual_file_ex(&new_state, filepath, NULL, realpath_mode)) {
+		efree(new_state.cwd);
 		return NULL;
 	}
 
@@ -801,7 +825,7 @@ PHPAPI char *expand_filepath_with_mode(const char *filepath, char *real_path, co
 	} else {
 		real_path = estrndup(new_state.cwd, new_state.cwd_length);
 	}
-	free(new_state.cwd);
+	efree(new_state.cwd);
 
 	return real_path;
 }
